MODELS -- Power Swing Analysis

 -- Interface from Power Network Cb 1
 -- Phase-to-ground voltages and phase currents
 -- Circuit-breaker Cb1
 -- primary circuit of the vt/ct
 INPUT VAG_M1 {TACS(VAG_M1)}
 INPUT VBG_M1 {TACS(VBG_M1)}
 INPUT VCG_M1 {TACS(VCG_M1)}
 INPUT IAP_M1 {TACS(IAP_M1)}
 INPUT IBP_M1 {TACS(IBP_M1)}
 INPUT ICP_M1 {TACS(ICP_M1)}

 -- secondary circuit of the vt/ct
 INPUT VAGSM1 {TACS(OUTP04)}
 INPUT VBGSM1 {TACS(OUTP05)}
 INPUT VCGSM1 {TACS(OUTP06)}
 INPUT IAPSM1 {TACS(OUTP01)}
 INPUT IBPSM1 {TACS(OUTP02)}
 INPUT ICPSM1 {TACS(OUTP03)}
 
 -- Input Interface at MODELS level: MODELS -> Power Network
 -- OUTPUT

 -- Start of the MODEL implementation (source code)
 MODEL PowerSwingImpedance

  -- Input Interface at MODEL level : MODELS -> MODEL source code
  INPUT
   VAG  -- Phase-to-ground voltage, primary
   VBG
   VCG
   IA   -- Phase currents, primary
   IB 
   IC 

   VAGsec  -- Phase-to-ground voltage, secondary
   VBGsec
   VCGsec
   IAsec   -- Phase currents, secondary
   IBsec 
   ICsec
   
  OUTPUT
   dRAG      -- dRAG / DT
   P1        -- Active power
   S1        -- Apparent power
   RAG       -- Resistance Re(VAG /IA)
   XAG       -- Reactance Im(VAG /IA)
   
  CONST PowerNetFreq {val: 50}  -- POWER FREQUENCY [Hz]
  CONST PowerNetCycle {val: 0.02}  -- Cycle [s]
  CONST SamplingFreq {val: 1000}  -- Sampling Frequency [Hz]
  CONST SamplingDT {val: 0.001}  -- Sampling Rate [s]
  CONST SamplesPerCycle {val: 20}  -- Samples per Cycle

  CONST
   Mega {val: 1.0E6}
   
  VAR
   RAG      -- Resistance impedance loop AG
   XAG      -- Reactance impedance loop AG
   dRAG     -- dR/dt
   
   -- Buffer RAG (k)
   vRAG[1..SamplesPerCycle]
   
   VAGre    -- Real part of fourier coefficient
   IAre
   VAGim    -- Imaginary part of fourier coefficient
   IAim
   
   P1, S1, S1del  -- Calculation of active power,
                  -- apparent power of Probe Prb1
   
   VAGdel
   IAdel
   VAGamount
   IAamount
   sq2divN
   h2pi_N
   Denominator
   Diff
   StartCounter
   k
      
  -- Berechnung des Momentanwertes der Scheinleistung
  FUNCTION CALCS(u, i) := 3.0 * (u * i) / Mega
 
  -- Berechnung der Wirkleistung als Mittelwert
  FUNCTION CALCP(P, S, Sdel, N) := (P + ((S - Sdel) / N))
   
  -- History buffer
  HISTORY
   VAG {dflt: 0.0}
   IA  {dflt: 0.0}
   VAGsec {dflt: 0.0}
   IAsec  {dflt: 0.0}
   
  -- Number of samples of the history buffers
  DELAY CELLS (VAG) : (PowerNetCycle / timestep) + 1
  DELAY CELLS (IA)  : (PowerNetCycle / timestep) + 1
  DELAY CELLS (VAGsec) : (PowerNetCycle / timestep) + 1
  DELAY CELLS (IAsec) : (PowerNetCycle / timestep) + 1
   
  -- One-time initialization of variables, etc. at simulation start
  INIT

   k := 0
   P1 := 0.0
   S1 := 0.0
   S1del := 0.0
   VAGre := 0.0
   VAGim := 0.0
   IAre := 0.0
   IAim := 0.0
   RAG := 0.0
   XAG := 0.0
   dRAG := 0.0
   VAGdel := 0.0
   IAdel := 0.0
   VAGamount := 0.0
   IAamount := 0.0
   Denominator := 0.0
   Diff := 0.0
   sq2divN := sqrt(2) / SamplesPerCycle
   h2pi_N := 2 * pi/ SamplesPerCycle
   
   StartCounter := 0
   
   FOR i := 1 TO SamplesPerCycle DO
    vRAG[i] := 0.0
   ENDFOR

  ENDINIT

  -- Execution at each simulation time step
  EXEC

   -- Active power, apparent power
   S1 := CALCS(VAG, IA)
   S1del := CALCS(delay(VAG, PowerNetCycle), delay(IA, PowerNetCycle))
   P1 := CALCP(P1, S1, S1del, SamplesPerCycle)
   
   write ("S1=", S1, " P1=", P1)
   
   -- DFT coefficient VAG
   VAGdel := delay(VAGsec, PowerNetCycle)                                          
   VAGre := VAGre + sq2divN * (VAGsec - VAGdel) * cos(k * h2pi_N) 
   VAGim := VAGim + sq2divN * (VAGsec - VAGdel) * sin(k * h2pi_N)

   -- DFT coefficient IA
   IAdel := delay(IAsec, PowerNetCycle)                                          
   IAre := IAre + sq2divN * (IAsec - IAdel) * cos(k * h2pi_N) 
   IAim := IAim + sq2divN * (IAsec - IAdel) * sin(k * h2pi_N)
   
   -- Amount and phase angle
   VAGamount := sqrt(VAGre * VAGre + VAGim * VAGim)
   IAamount := sqrt(IAre * IAre + IAim * IAim)
   
   -- Impedance ZA = VAG / IA berechnen
   Denominator := (IAre * IAre) + (IAim * IAim)
   RAG := ((VAGre * IAre) + (VAGim * IAim)) / Denominator
   XAG := ((VAGim * IAre) - (VAGre * IAim)) / Denominator
   
   -- Shift values in the buffer
   FOR i := 1 TO (SamplesPerCycle - 1) DO
    vRAG[i] := vRAG[i+1]
   ENDFOR
   vRAG[SamplesPerCycle] := RAG  -- add new value
   
   -- Soll die nderungsgeschwindigkeit berechnet werden ?
   IF (StartCounter > 0) THEN
    
    -- Intervall mit 2 Sttzstellen
    Diff := vRAG[SamplesPerCycle] - vRAG[SamplesPerCycle - 1]
    dRAG := Diff / timestep

   ELSE
    dRAG := 0.0
    StartCounter := StartCounter + 1
   ENDIF
   
   k := k + 1
   
  ENDEXEC

 ENDMODEL
 -- End of the MODEL implementation (source code)

 -- Start of the MODEL instance (task)
 USE PowerSwingImpedance AS PowerSwingImpedance

  -- Input - Argumentliste der Methode
  INPUT
   VAG := VAG_M1  -- Primre Leiter-Erd-Spannungen von MODELS
   VBG := VBG_M1
   VCG := VCG_M1
   IA  := IAP_M1  -- Primre Leiterstrme von MODELS
   IB  := IBP_M1
   IC  := ICP_M1

   VAGsec := VAGSM1  -- Sekundre Leiter-Erd-Spannungen von MODELS
   VBGsec := VBGSM1
   VCGsec := VCGSM1
   IAsec  := IAPSM1  -- Sekundre Leiterstrme von MODELS
   IBsec  := IBPSM1
   ICsec  := ICPSM1
   
 ENDUSE -- End of the MODEL instance (task)

 -- Recording signals, variables, etc, into the .PL4-file
 -- !!! PL4-Names max. 6 characters !!!
 RECORD
  PowerSwingImpedance.dRAG AS DRAG
  PowerSwingImpedance.S1 AS S1
  PowerSwingImpedance.P1 AS P1
  PowerSwingImpedance.RAG AS RAG
  PowerSwingImpedance.XAG AS XAG

ENDMODELS -- End of the MODELS Section
